#!/usr/bin/env python
DESC="""EPP used to create csv files for the nanoprep robot"""
import datetime
import logging
import re
import os

from argparse import ArgumentParser
from genologics.lims import Lims
from genologics.config import BASEURI,USERNAME,PASSWORD
from genologics.entities import *
from genologics.epp import attach_file


def generate_header(step,atype='D'):
    #generates an almost static header
    header="[Header]\nIEMFileVersion,4\nDate,{date}\nWorkflow,NeoPrep\nApplication,NeoPrep\n"\
    "Assay,TruSeq {atype}NA\nNotes,IEMTemplate\nOperator,{operator}\nRun Name,SciLifeTemplate\nChemistry Default\n\n[Reads]\n\n"\
    "[Settings]\n\n".format(date=datetime.datetime.now().strftime("%d/%m/%Y"), operator=step.technician.first_name, atype=atype)
    if atype == 'D':
        header+="[Data]\nSample_Name,Sample_Well,I7_Index_ID,index,Insert Size\n"
    else:
        header+="[Data]\nSample_Name,Sample_Well,I7_Index_ID,index\n"

    return header


def generate_data(step):
    logger=logging.getLogger(__name__)
    proto_pattern=re.compile("([3,5]50)")
    #contents of the rows will be taken from both input and output artifacts
    data=""
    for inout in step.input_output_maps:
        inp=inout[0]['uri']
        out=inout[1]['uri']
        if out.type == "Analyte":
            try:
                #samplename
                sname=out.samples[0].name
            except:
                logger.error("Cannot find the sample of output analyte {0}".format(out.id))
                return None

            try:
                #well number
                location=out.location[1]
                row=location.split(':')[0]
                col=location.split(':')[1]
                #neoprep expects wells to be 1-16, lims has A1-B8
                well=(ord(row)-64)+((int(col)-1)*8)#turns A1, B1 ... B8 into 1,2 ... 16
            except:
                logger.error("Cannot find the location of analyte {0}".format(out.id))
                return None
            try:
                #regent label (barcode) name and sequence
                reglab_name=out.reagent_labels[0]
                reglab_seq=lims.get_reagent_types(name=reglab_name)[0].sequence
            except:
                logger.error("Cannot find the reagent label of output analyte {0}".format(out.id))
                return None
            try:
                #insert size
                matches=proto_pattern.search(inp.udf['Covaris Protocol'])
                ins_size=matches.group(1)
            except:
                logger.error("Cannot find the insert size of analyte {0} ({1})".format(inp.id, sname))
                ins_size='350'


            if reglab_name == 'D':
                data+="{0},{1},{2},{3},{4}\n".format(sname, well, reglab_name, reglab_seq, ins_size)
            else:
                data+="{0},{1},{2},{3}\n".format(sname, well, reglab_name, reglab_seq)
    header=generate_header(step, reglab_name[1])
    return header+data
        


def generate_csv(lims,step_id, logfile):
    logger=setupLog(logfile)
    pro=Process(lims, id=step_id)
    data=generate_data(pro)
    with open("neoprep_input.csv", 'wb') as f:
        f.write(data)
    for out in pro.all_outputs():
        #attach the csv file 
        if out.name=="Input CSV File":
            attach_file(os.path.join(os.getcwd(), "neoprep_input.csv"), out)
        if out.name=="Log File":
            attach_file(os.path.join(os.getcwd(), logfile), out)

def setupLog(logfile):
        mainlog = logging.getLogger(__name__)
        mainlog.setLevel(level=logging.INFO)
        mfh = logging.FileHandler(logfile)
        mft = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        mfh.setFormatter(mft)
        mainlog.addHandler(mfh)
        return mainlog


if __name__ == "__main__":
    parser = ArgumentParser(description=DESC)
    parser.add_argument('--pid',
                        help='Lims id for current Process')
    parser.add_argument('--log', default="write_neoprep_csv.log",
                        help='logfile for this epp')
    args = parser.parse_args()

    lims = Lims(BASEURI, USERNAME, PASSWORD)
    lims.check_version()
    generate_csv(lims, args.pid, args.log)



#!/usr/bin/env python
DESC="""EPP used to create csv files for the bravo robot"""

from argparse import ArgumentParser
from genologics.lims import Lims
from genologics.config import BASEURI,USERNAME,PASSWORD
from genologics.epp import attach_file, EppLogger
import logging
import os
import sys
from genologics.entities import *


MAX_WARNING_VOLUME=150.0
MIN_WARNING_VOLUME=2.0

def obtain_previous_volumes(currentStep, lims):
    samples_volumes={}
    re_well = re.compile("([A-H]):?0?([0-9]{1,2})")
    previous_steps=set()
    for input_artifact in currentStep.all_inputs():
        previous_steps.add(input_artifact.parent_process)
    for pp in previous_steps:
        for output in pp.all_outputs():
            if output.name == "Normalization buffer volumes CSV":
                try:
                    fid=output.files[0].id
                except:
                    raise(RuntimeError("Cannot access the normalisation CSV file to read the volumes."))
                else:
                    file_contents=lims.get_file_contents(id=fid)
                    genologics_format = False
                    well_idx = 4
                    plate_idx = 3
                    source_vol_idx = 2
                    buffer_vol_idx = 5
                    for line in file_contents.split('\n'):
                        # Skip some lines:
                        if not line.rstrip():
                            continue
                        elif "Date of file generation:" in line:
                            continue
                        elif "Generated by:" in line:
                            continue
                        elif "Sample Name" in line:
                            # This is Genologics format and the header line
                            # so change column indices:
                            genologics_format = True
                            elements=line.split(',')
                            for idx, el in enumerate(elements):
                                if el == "Source Volume (uL)":
                                    source_vol_idx=idx
                                elif el == "Volume of Dilution Buffer (uL)":
                                    buffer_vol_idx=idx
                                elif el == "Destination Well":
                                    well_idx=idx
                                elif el == "Destination Plate":
                                    plate_idx=idx
                                elif el == "Sample Name":
                                    name_idx=idx
                        else:
                            elements=line.split(',')
                            well = elements[well_idx]
                            matches = re_well.search(well)
                            if matches:
                                well = ":".join(x for x in matches.groups())
                            plate = elements[plate_idx]
                            srcvol = elements[source_vol_idx]
                            bufvol = elements[buffer_vol_idx]
                            # Remove any quotes:
                            (plate, srcvol, bufvol) = [s.replace('"','') for s in (plate, srcvol, bufvol)]
                            srcvol=float(srcvol)
                            bufvol=float(bufvol)
                            totvol = bufvol
                            # For Genologics format compability:
                            if genologics_format:
                                totvol += srcvol
                                name = elements[name_idx].replace('"','')
                                samples_volumes[name] = totvol
                            else:
                                samples_volumes.setdefault(plate,{})[well] = totvol
    return samples_volumes



def make_datastructure(currentStep, lims, log):
    data=[]
    samples_volumes={}
    try:
        samples_volumes=obtain_previous_volumes(currentStep, lims)
    except:
        log.append("Unable to find previous volumes")
        sys.stderr.write("Samples volumes cannot be found. Check the file from the previous step")
        sys.exit(2)

    for inp, out in currentStep.input_output_maps:
        if out['output-type'] == 'Analyte':
            obj={}
            obj['name']=inp['uri'].samples[0].name
            obj['id']=inp['uri'].id
            obj['conc']=inp['uri'].udf['Normalized conc. (nM)']
            obj['pool_id']=out['uri'].id
            #obj['pool_conc']=out['uri'].udf['Normalized conc. (nM)']
            obj['src_fc']=inp['uri'].location[0].name
            obj['src_fc_id']=inp['uri'].location[0].id
            obj['src_well']=inp['uri'].location[1]
            obj['dst_fc']=out['uri'].location[0].id
            obj['dst_well']=out['uri'].location[1]
            # For Genologics format compability:
            if obj['name'] in samples_volumes:
                obj['vol'] = samples_volumes[obj['name']]
            # Try to match container ID first then name:
            elif obj['src_fc_id'] in samples_volumes:
                obj['vol'] = samples_volumes[obj['src_fc_id']][obj['src_well']]
            else:
                obj['vol'] = samples_volumes[obj['src_fc']][obj['src_well']]
            data.append(obj)

    return data

######### LAZY WAY ###############################
# Just divide the total with the number of samples, it is implied that the final
# conc and the conc of every input is the same:
def lazy_volumes(samples, final_vol):
    return [final_vol / len(samples) for s in samples]

######### OTHER WAY ##############################
# Iteratively reduce the smallest input volume until we are close to the desired
# total volume. This works when the inputs have different concentrations, the
# final pool concentration will then end up somewhere between the conc of the
# highest and the lowest input concentration:
def optimize_volumes(samples, final_vol, limit_vol=2):
    # Create a list we can sort to get the min/max values:
    l = [(s["conc"]*s["vol"],s["conc"],s["vol"]) for s in samples]
    # Find the min/max values by sorting on the different values:
    min_conc = sorted(l, key=lambda x: x[1])[0][1]
    max_conc = sorted(l, key=lambda x: x[1])[-1][1]
    min_vol = sorted(l, key=lambda x: x[2])[0][2]
    # The volume of the input with lowest amount:
    min_amount = sorted(l)[0][2]

    def _minimize_vol(vol, final_vol=final_vol, limit_vol=limit_vol, reduce=0.9):
        try_vol = reduce * vol
        # The lowest volume to take would then be (sample(s) w highest conc):
        low_vol = min(try_vol * max_conc / s["conc"] for s in samples)
        # Total pool volume if we were to take this amount of all samples:
        tot_vol = sum(try_vol * max_conc / s["conc"] for s in samples)
        # We don't want to pipette less than limit_vol
        # while keeping total volume above final_vol:
        if low_vol >= limit_vol and tot_vol >= final_vol and try_vol >= limit_vol:
            return _minimize_vol(try_vol)
        else:
            # We can't improve anymore within the given limits...
            return vol

    # Start from whichever is the smallest volume:
    use_vol = _minimize_vol(min(min_amount, min_vol))
    # Calculate the volume to take of each input:
    return [(use_vol * max_conc / s["conc"]) for s in samples]

def compute_transfer_volume(currentStep, lims, log):
    data=make_datastructure(currentStep, lims, log)
    returndata=[]
    for pool in currentStep.all_outputs():
        if pool.type == 'Analyte':
            valid_inputs=filter(lambda x: x['pool_id']==pool.id, data)
            # Set the output conc of the pool and also get the "desired" pool
            # volume, which is which?
            final_vol = float(pool.udf["Final Volume (uL)"])
            conc = valid_inputs[0]["conc"]
            # If all inputs are of the same conc use the trivial algorithm,
            # else try to optimize:
            if all(s["conc"] == conc for s in valid_inputs):
                vols = lazy_volumes(valid_inputs, final_vol)
                pool.udf['Normalized conc. (nM)'] = conc
            else:
                vols = optimize_volumes(valid_inputs, final_vol, MIN_WARNING_VOLUME)
                # Calculate and add the theoretical pool conc:
                z = zip([s["conc"] for s in valid_inputs], vols)
                v = (sum(x[0]*x[1] for x in z) / sum(vols))
                pool.udf['Normalized conc. (nM)'] = v
            pool.put()
            for s, vol in zip(valid_inputs, vols):
                s['vol_to_take'] = vol
                returndata.append(s)

    return returndata

def prepooling(currentStep, lims):
    log=[]
    #First thing to do is to grab the volumes of the input artifacts. The method is ... rather unique.
    data=compute_transfer_volume(currentStep, lims, log)
    with open("bravo.csv", "w") as csvContext:
            for s in data:
                if s['vol_to_take']>MAX_WARNING_VOLUME:
                    log.append("Volume for sample {} is above {}, redo the calculations manually".format(MAX_WARNING_VOLUME, s['name']))
                if s['vol_to_take']<MIN_WARNING_VOLUME:
                    log.append("Volume for sample {} is below {}, redo the calculations manually".format(MIN_WARNING_VOLUME, s['name']))
                csvContext.write("{0},{1},{2},{3},{4}\n".format(s['src_fc_id'], s['src_well'], s['vol_to_take'], s['dst_fc'], s['dst_well']))
    if log:
        with open("bravo.log", "w") as logContext:
            logContext.write("\n".join(log))
    for out in currentStep.all_outputs():
        #attach the csv file and the log file
        if out.name=="Bravo CSV File":
            attach_file(os.path.join(os.getcwd(), "bravo.csv"), out)
        if log and out.name=="Bravo Log":
            attach_file(os.path.join(os.getcwd(), "bravo.log"), out)
    if log:
        #to get an eror display in the lims, you need a non-zero exit code AND a message in STDERR
        sys.stderr.write("Errors were met, please check the Log file\n")
        sys.exit(2)
    else:
        logging.info("Work done")

def setup_workset(currentStep):
    checkTheLog=[False]
    with open("bravo.csv", "w") as csvContext:
        with open("bravo.log", "w") as logContext:
            #working directly with the map allows easier input/output handling
            for art_tuple in currentStep.input_output_maps:
                #filter out result files
                if art_tuple[0]['uri'].type=='Analyte' and art_tuple[1]['uri'].type=='Analyte':
                    source_fc=art_tuple[0]['uri'].location[0].name
                    source_well=art_tuple[0]['uri'].location[1]
                    dest_fc=art_tuple[1]['uri'].location[0].id
                    dest_well=art_tuple[1]['uri'].location[1]
                    try:
                        #might not be filled in
                        final_volume=art_tuple[1]['uri'].udf["Total Volume (uL)"]
                    except KeyError as e:
                        logContext.write("No Total Volume found for sample {0}\n".format(art_tuple[0]['uri'].samples[0].name))
                        checkTheLog[0]=True
                    else:
                        volume=calc_vol(art_tuple, logContext, checkTheLog)
                        csvContext.write("{0},{1},{2},{3},{4},{5}\n".format(source_fc, source_well, volume, dest_fc, dest_well, final_volume))
    for out in currentStep.all_outputs():
        #attach the csv file and the log file
        if out.name=="Bravo CSV File":
            attach_file(os.path.join(os.getcwd(), "bravo.csv"), out)
        if out.name=="Bravo Log":
            attach_file(os.path.join(os.getcwd(), "bravo.log"), out)
    if checkTheLog[0]:
        #to get an eror display in the lims, you need a non-zero exit code AND a message in STDERR
        sys.stderr.write("Errors were met, please check the Log file\n")
        sys.exit(2)
    else:
        logging.info("Work done")

def normalization(current_step):
    log = []
    with open("normalization.csv", "w") as csv:
        for art in current_step.input_output_maps:
            src = art[0]["uri"]
            dest = art[1]["uri"]
            if src.type == dest.type == "Analyte":
                # Source sample:
                src_plate = src.location[0].id
                src_well = src.location[1]
                src_tot_volume = float(src.udf["Volume (ul)"])
                src_volume = float(dest.udf["Volume to take (uL)"])
                src_conc = src.udf["Concentration"]
                # Diluted sample:
                dest_plate = dest.location[0].id
                dest_well = dest.location[1]
                dest_conc = dest.udf["Normalized conc. (nM)"]
                if src.udf["Conc. Units"] != "nM":
                    log.append("ERROR: No valid concentration found for sample {0}".format(src.samples[0].name))
                elif src_conc < dest_conc:
                    log.append("ERROR: Too low concentration for sample {0}".format(src.samples[0].name))
                else:
                    # Warn if volume to take > volume available or max volume is
                    # exceeded but still do the calculation:
                    if src_volume > src_tot_volume:
                        log.append("WARNING: Not enough available volume of sample {0}".format(src.samples[0].name))
                    final_volume = src_conc * src_volume / dest_conc
                    if final_volume > MAX_WARNING_VOLUME:
                        log.append("WARNING: Maximum volume exceeded for sample {0}".format(src.samples[0].name))
                    csv.write("{0},{1},{2},{3},{4},{5}\n".format(src_plate, src_well, src_volume, dest_plate, dest_well, final_volume))
    if log:
        with open("normalization.log", "w") as log_context:
            log_context.write("\n".join(log))
    for out in current_step.all_outputs():
        #attach the csv file and the log file
        if out.name == "Normalization buffer volumes CSV":
            attach_file(os.path.join(os.getcwd(), "normalization.csv"), out)
        elif out.name == "Normalization Log" and log:
            attach_file(os.path.join(os.getcwd(), "normalization.log"), out)
    if log:
        #to get an eror display in the lims, you need a non-zero exit code AND a message in STDERR
        sys.stderr.write("Errors were met, please check the log file\n")
        sys.exit(2)
    else:
        logging.info("Work done")

def main(lims, args):
    #Array, so can be modified inside a child method
    currentStep=Process(lims,id=args.pid)
    if "Setup" in currentStep.type.name or "Genotyping" in currentStep.type.name:
        setup_workset(currentStep)
    elif "Pooling" in currentStep.type.name:
        prepooling(currentStep, lims)
    elif "Normalization" in currentStep.type.name:
        normalization(currentStep)

def calc_vol(art_tuple, logContext,checkTheLog):
    try:
        #not handling different units yet. Might be needed at some point.
        assert art_tuple[0]['uri'].udf['Conc. Units'] == "ng/ul"
        amount_ng=art_tuple[1]['uri'].udf['Amount taken (ng)']
        conc=art_tuple[0]['uri'].udf['Concentration']
        volume=float(amount_ng)/float(conc)
        if volume<MIN_WARNING_VOLUME:
            #arbitrarily determined by Sverker Lundin
            logContext.write("WARN : Sample {0} located {1} {2}  has a LOW volume : {3}\n".format(art_tuple[1]['uri'].samples[0].name,
                art_tuple[0]['uri'].location[0].name,art_tuple[0]['uri'].location[1], volume))
            checkTheLog[0]=True
        elif volume>art_tuple[0]['uri'].udf["Volume (ul)"]:
            #check against the "max volume"
            logContext.write("WARN : Sample {0} located {1} {2}  has a HIGH volume : {3}, over {4}\n".format(art_tuple[1]['uri'].samples[0].name,
                art_tuple[0]['uri'].location[0].name, art_tuple[0]['uri'].location[1], volume,art_tuple[0]['uri'].udf["Volume (ul)"] ))
            checkTheLog[0]=True
        elif volume>art_tuple[1]['uri'].udf['Total Volume (uL)']:
            logContext.write("WARN : Sample {0} located {1} {2}  has a HIGHER volume than the total: {3}, over {4}\n".format(art_tuple[1]['uri'].samples[0].name,
                art_tuple[0]['uri'].location[0].name, art_tuple[0]['uri'].location[1], volume,art_tuple[1]['uri'].udf["Total Volume (uL)"] ))
            checkTheLog[0]=True
        else:
            logContext.write("INFO : Sample {0} looks okay.\n".format(art_tuple[1]['uri'].samples[0].name))
        return "{0:.2f}".format(volume)
    except KeyError as e:
        logContext.write("ERROR : The input artifact is lacking a field : {0}\n".format(e))
        checkTheLog[0]=True
    except AssertionError:
        logContext.write("ERROR : This script expects the concentration to be in ng/ul, this does not seem to be the case.\n")
        checkTheLog[0]=True
    #this allows to still write the file. Won't be readable though
    return "#ERROR#"

if __name__=="__main__":
    parser = ArgumentParser(description=DESC)
    parser.add_argument('--pid',
                        help='Lims id for current Process')
    args = parser.parse_args()

    lims = Lims(BASEURI, USERNAME, PASSWORD)
    lims.check_version()
    main(lims, args)




from genologics.entities import *
from genologics.lims import *
from genologics.config import BASEURI, USERNAME, PASSWORD




lims= Lims(BASEURI, USERNAME, PASSWORD)


def  get_udfs():

    step_name='End repair, A-tailing and adapter ligation (TruSeq RNA) 4.0'
    udf_name="Lot no: Adaptor Plate"

    pros=lims.get_processes(type=step_name)
    for pro in pros:
        try:
            print "{}\t{} {}\t{}".format(pro.date_run,pro.technician.first_name,pro.technician.last_name ,pro.udf[udf_name] )
        except:
            print "{}\t{} {}".format(pro.date_run,pro.technician.first_name,pro.technician.last_name)

get_udfs()



import sys
import re
import fileinput
import csv
import os

from argparse import ArgumentParser
from genologics.lims import Lims
from genologics.config import BASEURI,USERNAME,PASSWORD
from genologics.epp import attach_file, EppLogger
from genologics.entities import *

DESC="""EPP used to parse csv files from the Tecan plate reader"""

def parse(iterable):
    # Pattern for matching "Layout" ID:
    p = re.compile(r"(S[MT])1_(\d{1,2})")
    # Read in the csv file:
    for line in iterable.splitlines():
        if "#" in line:
            vals = line.rstrip().replace("#","").replace(" ","").split(",")
            vals.append("#")
        else:
            vals = line.rstrip().replace(" ","").split(",")
            vals.append("")
        # Filter out all rows with sample and standard entries and
        # only use the "1/3" replicate:
        m = p.match(vals[1])
        if m and vals[2] == "1/3":
            # Substitute replicate with index:
            vals[2] = m.group(2)
            # Assign a type and source well:
            if m.group(1) == "SM":
                vals[1] = "Sample"
                vals[0] = index_to_well(m.group(2))
            else:
                vals[1] = "Standard"
                vals[0] = ""
            # Skip some columns:
            del vals[9]
            del vals[6]
            del vals[3]
            #yield([vals[i] for i in xrange(0,9)])
            yield vals

def convert(file_in, file_out):
    # Make a list of it to be able to sort:
    data = list(parse(file_in))
    # Sort the results on type then index:
    data.sort(key=lambda x: (x[1], int(x[2])))
    # Create the header:
    header = [
        "Well",
        "Type",
        "Index",
        "Raw mean",
        "Raw CV%",
        "Conc mean (ng/uL)",
        "Conc CV%",
        "Mark"
    ]
    # Write data:
    writer = csv.writer(file_out)
    writer.writerow(header)
    writer.writerows(data)

    return data

# Convert an index to a plate coordinate, e.g. 8 => H1
def index_to_well(index):
    i = int(index)
    row = chr(64 + ((i-1) % 8 + 1))
    col = int(((i-1) - (i-1) % 8) / 8 + 1)
    return "{0}{1}".format(row, col)

def dictionnarize(datalist):
    data_to_upload={}
    for row in datalist:
        data_to_upload[row[0]]={}
        data_to_upload[row[0]]['conc']=row[5]
        data_to_upload[row[0]]['cv']=row[6]

    return data_to_upload




def main(args, lims):
    pro=Process(lims, id=args.pid)
    for output in pro.all_outputs():
        if output.name == "Tecan output file":
            try:
                fid=output.files[0].id
            except:
                raise(RuntimeError("Cannot access the tecan output file to read the concentrations."))
        elif output.name=='EPP log file':
            out_id=output.id

    file_contents=lims.get_file_contents(id=fid)
    with open('{0}_tecan.out'.format(out_id), 'w') as outf:
        data=convert(file_contents, outf)
                
    di=dictionnarize(data)

    for iom in pro.input_output_maps:
        if iom[1]['uri'].output_type == "ResultFile" and len(iom[1]['uri'].samples) ==1:
            if 'P2656_243' in [x.name for x in iom[1]['uri'].samples]:
                import pdb;pdb.set_trace()
            poskey=iom[1]['uri'].location[1].replace(":", "")
            iom[1]['uri'].udf['Conc. Units']='ng/ul'
            iom[1]['uri'].udf['Concentration']=float(di[poskey]['conc'])
            iom[1]['uri'].udf['%CV']=float(di[poskey]['cv'])
            iom[1]['uri'].put()

if __name__ == "__main__":
    parser = ArgumentParser(description=DESC)
    parser.add_argument('--pid',
                        help='Lims id for current Process')
    parser.add_argument('--log',
                        help='log file name')
    args = parser.parse_args()

    lims = Lims(BASEURI, USERNAME, PASSWORD)
    main(args, lims)



#!/usr/bin/env python
DESC="""EPP script to calculate amount in ng from concentration and volume 
udf:s in Clarity LIMS. The script checks that the 'Volume (ul)' and 
'Concentration' udf:s are defined and that the udf. 'Conc. Units' 
 have the correct value: 'ng/ul', otherwise that artifact is skipped, 
left unchanged, by the script.

Johannes Alneberg, Science for Life Laboratory, Stockholm, Sweden
""" 
from argparse import ArgumentParser

from genologics.lims import Lims
from genologics.config import BASEURI,USERNAME,PASSWORD

from genologics.entities import Process
from genologics.epp import EppLogger

import logging
import sys

def apply_calculations(lims,artifacts,udf1,op,udf2,result_udf,epp_logger,process):
    """For each result file of the process: if its corresponding inart has the udf 
    'Dilution Fold', the result_udf: 'Amount (ng)' is calculated as
   
    'Amount (ng)' =  'Concentration'*'Volume (ul)'*'Dilution Fold'
    
    otherwise its calculated as
        
    'Amount (ng)' =  'Concentration'*'Volume (ul)'"""

    logging.info(("result_udf: {0}, udf1: {1}, "
                  "operator: {2}, udf2: {3}").format(result_udf,udf1,op,udf2))
    for artifact in artifacts:
        try:
            artifact.udf[result_udf]
        except KeyError:
            artifact.udf[result_udf]=0

        try:
            inart = process.input_per_sample(artifact.samples[0].name)[0]
            dil_fold = inart.udf['Dilution Fold']
        except:
            dil_fold = None

        logging.info(("Updating: Artifact id: {0}, "
                     "result_udf: {1}, udf1: {2}, "
                     "operator: {3}, udf2: {4}").format(artifact.id, 
                                                        artifact.udf.get(result_udf,0),
                                                        artifact.udf[udf1],op,
                                                        artifact.udf[udf2]))
        prod = eval('{0}{1}{2}'.format(artifact.udf[udf1],op,artifact.udf[udf2]))
        if dil_fold:
            prod = eval('{0}{1}{2}'.format(prod, op, dil_fold))
        artifact.udf[result_udf] = prod
        artifact.put()
        logging.info('Updated {0} to {1}.'.format(result_udf,
                                                 artifact.udf[result_udf]))
            
def check_udf_is_defined(artifacts, udf):
    """ Filter and Warn if udf is not defined for any of artifacts. """
    filtered_artifacts = []
    incorrect_artifacts = []
    for artifact in artifacts:
        if (udf in artifact.udf):
            filtered_artifacts.append(artifact)
        else:
            logging.warning(("Found artifact for sample {0} with {1} "
                             "undefined/blank, skipping").format(artifact.samples[0].name, udf))
            incorrect_artifacts.append(artifact)
    return filtered_artifacts, incorrect_artifacts


def check_udf_has_value(artifacts, udf, value):
    """ Filter artifacts on undefined udf or if udf has wrong value. """
    filtered_artifacts = []
    incorrect_artifacts = []
    for artifact in artifacts:
        if udf in artifact.udf and (artifact.udf[udf] == value):
            filtered_artifacts.append(artifact)
        elif udf in artifact.udf:
            incorrect_artifacts.append(artifact)
            logging.warning(("Filtered out artifact for sample: {0}"
                          ", due to wrong {1}").format(artifact.samples[0].name, udf))
        else:
            incorrect_artifacts.append(artifact)
            logging.warning(("Filtered out artifact for sample: {0}"
                          ", due to undefined/blank {1}").format(artifact.samples[0].name, udf))

    return filtered_artifacts, incorrect_artifacts

def main(lims,args,epp_logger):
    p = Process(lims,id = args.pid)
    udf_check = 'Conc. Units'
    value_check = 'ng/ul'
    udf_factor1 = 'Concentration'
    udf_factor2 = 'Volume (ul)'
    result_udf = 'Amount (ng)'

    if args.aggregate:
        artifacts = p.all_inputs(unique=True)
    else:
        all_artifacts = p.all_outputs(unique=True)
        artifacts = filter(lambda a: a.output_type == "ResultFile" ,all_artifacts)

    correct_artifacts, wrong_factor1 = check_udf_is_defined(artifacts, udf_factor1)
    correct_artifacts, wrong_factor2 = check_udf_is_defined(correct_artifacts, udf_factor2)

    correct_artifacts, wrong_value = check_udf_has_value(correct_artifacts, udf_check, value_check)

    if correct_artifacts:
        apply_calculations(lims, correct_artifacts, udf_factor1, '*',
                           udf_factor2, result_udf, epp_logger, p)

    d = {'ca': len(correct_artifacts),
         'ia': len(wrong_factor1)+ len(wrong_factor2) + len(wrong_value)}

    abstract = ("Updated {ca} artifact(s), skipped {ia} artifact(s) with "
                "wrong and/or blank values for some udfs.").format(**d)

    print >> sys.stderr, abstract # stderr will be logged and printed in GUI


if __name__ == "__main__":
    # Initialize parser with standard arguments and description
    parser = ArgumentParser(description=DESC)
    parser.add_argument('--pid',
                        help='Lims id for current Process')
    parser.add_argument('--log',
                        help='Log file for runtime info and errors.')
    parser.add_argument('--aggregate', action='store_true',
                        help=('Use this tag if current Process is an '
                              'aggregate QC step'))
    args = parser.parse_args()

    lims = Lims(BASEURI, USERNAME, PASSWORD)
    lims.check_version()

    with EppLogger(args.log, lims=lims, prepend=True) as epp_logger:
        main(lims, args, epp_logger)



#!/usr/bin/env python
DESC = """This EPP script reads Application QC files from file
system and sets the qc values for each sample. Allso a a easy to read App QC 
file is generated with more information about the application specific qc that 
was done. The file is suposed to be a help file in case of failed QC -flaggs.

Logging:
    The script outputs a regular log file with regular execution information.

Written by Maya Brandi (14-10-14)
"""

import os
import sys
import logging
import glob
import csv
import json

from argparse import ArgumentParser
from genologics.lims import Lims
from genologics.config import BASEURI, USERNAME, PASSWORD
from genologics.entities import Process
from genologics.epp import EppLogger
from genologics.epp import set_field

class AppQC():
    def __init__(self, process):
        self.app_QC = {}
        self.project_name = process.all_outputs()[0].samples[0].project.name
        self.target_files = dict((r.samples[0].name, r) for r in process.all_outputs())
        self.missing_samps = []
        self.nr_samps_updat = 0
        self.abstract = []
        self.process = process
        self.nr_samps_tot = str(len(self.target_files))
        self.QF_from_file = {}

    def get_app_QC_file(self):
        """ App QC file is read from the file msf system. Path hard coded."""
        file_path = ("/srv/mfs/app_QC/{0}.json".format(self.project_name))
        json_data = open(file_path).read()
        self.app_QC = json.loads(json_data)

    def set_result_file_udfs(self):
        """populates the target file App QC udf"""
        for samp_name, target_file in self.target_files.items():
            if samp_name in self.app_QC.keys():
                qc_passed = str(self.app_QC[samp_name]['automated_qc']['qc_passed'])
                sample = target_file.samples[0]
                sample.udf['App QC'] = qc_passed
                set_field(sample)
                self.nr_samps_updat += 1
            else:
                self.missing_samps.append(samp_name)
    
    def make_App_QC_file(self, app_qc_file):
        """Formates a easy to read App QC file."""
        keys= ['sample', 'qc_passed', 'qc_reason']
        list2csv = []
        for samp, info in self.app_QC.items():
            d = info['automated_qc']
            d['sample'] = samp
            list2csv.append(d)
        app_qc_file = app_qc_file + '.csv'
        test_file = open(app_qc_file,'wb')
        dict_writer = csv.DictWriter(test_file, keys, dialect = 'excel')
        dict_writer.writer.writerow(keys)
        dict_writer.writerows(list2csv)
        test_file.close()

    def logging(self):
        """Collects and prints logging info."""
        self.abstract.append("qc-flaggs uploaded for {0} out of {1} samples."
                                        "See App_QC_file for details.".format(
                                        self.nr_samps_updat, self.nr_samps_tot))
        if self.missing_samps:
            self.abstract.append("The following samples are missing in "
                                                    "App_QC_file: {0}.".format(
                                                ', '.join(self.missing_samps)))
        print >> sys.stderr, ' '.join(self.abstract)

def main(lims, pid, epp_logger, App_QC_file):
    process = Process(lims, id = pid)
    AQC = AppQC(process)
    AQC.get_app_QC_file()
    AQC.set_result_file_udfs()
    AQC.make_App_QC_file(App_QC_file)
    AQC.logging()
    

if __name__ == "__main__":
    parser = ArgumentParser(description=DESC)
    parser.add_argument('--pid', default = None , dest = 'pid',
                        help='Lims id for current Process')
    parser.add_argument('--log', dest = 'log',
                        help=('File name for standard log file, '
                              'for runtime information and problems.'))
    parser.add_argument('--file', dest = 'file',
                        help=('File path to new App_QC file'))
    args = parser.parse_args()
    lims = Lims(BASEURI, USERNAME, PASSWORD)
    lims.check_version()

    with EppLogger(log_file=args.log, lims=lims, prepend=True) as epp_logger:
        main(lims, args.pid, epp_logger, args.file)
